/*
 *  ProfileRotate.c
 *  DrawExamples
 *
 *  Created by Simone Portuesi on 08/03/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

/* Università Roma 3, DIA, Informatica Grafica Esercitazione del 8/3/2006 */

/* Modello esempio 2: Rotazione profile                                   */

/* Include GL, GLU & GLUT */
#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#endif

#ifdef linux
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

/* Include some standard stuff */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Include my files */
#include "main.h"
#include "utility.h"
#include "TexUtils.h"
int shownormals=0;

struct Texture {
	GLuint texName;
	char *filename;
	int w;
	int h;
};

struct Texture textures[] = { { 0, "checker.ppm" , 0, 0 },
							  { 0, "camuflage.ppm", 0, 0 },
							  { 0, "flatland.ppm", 0, 0 },
							  { 0, "logo.ppm", 0, 0 },
							  { 0, "swirl.ppm", 0, 0 },
							  { 0, "twirl.ppm", 0, 0 },
							  { 0, "heightmap.ppm", 0, 0 },
							  { 0, "land.ppm", 0, 0 },
							  { 0, "tunket.ppm", 0, 0 },
							  { 0, "marble.ppm", 0, 0 },
							  { 0, "fishermen.ppm", 0, 0 },
							  { 0, "marble.ppm", 0, 0 },
							  { 0, "opengl.ppm", 0, 0 } };
int n_textures=13;
int cur_texture=0;

enum { TxV, LINEAR, LINEAR_EYE, LIN_S_EYE_T, EYE_S_LIN_T, SPHERE } tex_mode=TxV;

void select_tex_mode() {
	GLfloat xy_plane[]={ 0, 0, 1, -1 };
	GLfloat xz_plane[]= { 0, 1, 0, 1 };
	switch(tex_mode) {
		case TxV:
			glDisable(GL_TEXTURE_GEN_S);
			glDisable(GL_TEXTURE_GEN_T);
			break;
		case LINEAR:
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
			glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
			glTexGenfv(GL_S,GL_OBJECT_PLANE,xz_plane);
			glTexGenfv(GL_T,GL_OBJECT_PLANE,xy_plane);
			break;
		case LINEAR_EYE:	
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
			glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
			glTexGenfv(GL_S,GL_OBJECT_PLANE,xz_plane);
			glTexGenfv(GL_T,GL_OBJECT_PLANE,xy_plane);
			break;
		case LIN_S_EYE_T:
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
			glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
			glTexGenfv(GL_S,GL_OBJECT_PLANE,xz_plane);
			glTexGenfv(GL_T,GL_OBJECT_PLANE,xy_plane);
			break;
		case EYE_S_LIN_T:
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
			glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_OBJECT_LINEAR);
			glTexGenfv(GL_S,GL_OBJECT_PLANE,xz_plane);
			glTexGenfv(GL_T,GL_OBJECT_PLANE,xy_plane);
			break;
		case SPHERE:
			glEnable(GL_TEXTURE_GEN_S);
			glEnable(GL_TEXTURE_GEN_T);
			glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
			glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
			
	}
}

void deselect_tex_mode() {
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
}

void rotateprofile(GLfloat profile[][2], int n_profile, int n) {
	int i,j,k;
	GLfloat angle0,angle1,angle1r;
	GLfloat step=2.0 * M_PI / n;
	GLfloat profileN[2];
	GLfloat d;
		
	/* For each segment */
	for(i=0;i<n_profile-1;i++) {
		/* For each slice */
		for(j=0;j<n;j++) {
			/* Calculate angles */
			angle0=j * step;
			angle1=((j+1) % n) * step;
			angle1r=(j+1) * step;
			
			/* Calculate normal of profile */
			profileN[0] = profile[i][1] -  profile[i+1][1];
			profileN[1] = profile[i+1][0] -  profile[i][0];
			d=sqrt(profileN[0] * profileN[0] + profileN[1] * profileN[1]);
			profileN[0] /=d;
			profileN[1] /=d;
			
			/* Calculate profile distance from 0 */
			if(shownormals) {
				glDisable(GL_LIGHTING);
				glBegin(GL_LINES);
				glVertex3f(profile[i][0] * cos(angle0),
						profile[i][1] ,
						profile[i][0] * sin(angle0));
				glVertex3f((profile[i][0] + profileN[0])  * cos(angle0),
						profile[i][1] + profileN[1],
						(profile[i][0] + profileN[0])  * sin(angle0) );
			
				glEnd();
				glBegin(GL_LINES);
				glVertex3f(profile[i+1][0] * cos(angle1),
						profile[i+1][1] ,
						profile[i+1][0] * sin(angle1));
				glVertex3f((profile[i+1][0] + profileN[0])  * cos(angle1),
						profile[i+1][1] + profileN[1],
						(profile[i+1][0] + profileN[0])  * sin(angle1) );
			
				glEnd();
				glEnable(GL_LIGHTING);
			}
			
			/* Enable Textures */
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, textures[cur_texture].texName);
			glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
			select_tex_mode();
			
			/* Draw face, updating the angle */
			glBegin(GL_POLYGON);
			glPolygonMode(GL_FRONT,GL_FILL);
			
			/* first point of profile projected on first angle */
			glTexCoord2f(angle0 / (2 * M_PI),
						 1 - ((GLfloat) i)/((GLfloat) n_profile));
			glNormal3f(profileN[0] * cos(angle0),
					   profileN[1],
					   profileN[0] * sin(angle0));
			glVertex3f( profile[i][0] * cos(angle0),
			            profile[i][1],
						profile[i][0] * sin(angle0) );
						
			/* second point of profile projected on first angle */
			glTexCoord2f(angle0 / (2 * M_PI),
						 1 - ((GLfloat) (i+1))/((GLfloat) n_profile));
			glNormal3f(profileN[0] * cos(angle0),
					   profileN[1],
					   profileN[0] * sin(angle0));
			glVertex3f( profile[i+1][0] * cos(angle0),
			            profile[i+1][1],
						profile[i+1][0] * sin(angle0) );
									
			/* second point of profile projected on second angle */		
			glTexCoord2f(angle1r / (2 * M_PI),
						 1 - ((GLfloat) (i+1))/((GLfloat) n_profile));				
			glNormal3f(profileN[0] * cos(angle1),
					   profileN[1],
					   profileN[0] * sin(angle1));
			glVertex3f( profile[i+1][0] * cos(angle1),
			            profile[i+1][1],
						profile[i+1][0] * sin(angle1));
			
			/* first point of profile projected on second angle */
			glTexCoord2f(angle1r / (2 * M_PI),
						 1 - ((GLfloat) i)/((GLfloat) n_profile));	
			glNormal3f(profileN[0] *  cos(angle1),
					   profileN[1],
					   profileN[0]*  sin(angle1));
			glVertex3f( profile[i][0] * cos(angle1),
			            profile[i][1],
						profile[i][0] * sin(angle1));
			
			glEnd();
			
			
			/* Disable textures */
			deselect_tex_mode();
			glDisable(GL_TEXTURE_2D);

		}
	}
} 


/* Current model */
void display_model() {
	GLfloat light_pos[]={ -1.0, 2.0, 3.0 };
	GLfloat light_pos1[]={ -1.0, -3.0, 0.0 };
	GLfloat profile[][2] = { {0.5, 1    }, 
							 {0.7, 0.8  }, 
							 {0.85, 0.6  },
							 {0.95, 0.4  },  
							 {0.97 ,0.2}, 
							 {1  , 0    }, 
							 {0.95, -0.2  },  
							 {0.85, -0.4  }, 
							 {0.85, -0.6 },  
							 {0.95, -0.8  }, 
							 {1,   -1   } };
	int n_profile=11;
		
	/* Fiat Lux */	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos );
	glEnable(GL_LIGHT1);
	glLightfv(GL_LIGHT1, GL_POSITION, light_pos1 );
	
	/* Make profile */
	glColor3f(1,1,1);
	rotateprofile(profile,n_profile,16);
	
	/* Turn off lighting */
	glDisable(GL_NORMALIZE);
	glDisable(GL_LIGHTING);
}


/* Model specific keys */
void keyboard_model(unsigned char key, int x, int y) {
	switch(key) {
		case 'n':
		case 'N':
			shownormals=!shownormals;
			glutPostRedisplay();
			break;
		case 't':
		case 'T':
			cur_texture=(cur_texture+1) % n_textures;
			glutPostRedisplay();
			break;
		case 'h':
		case 'H':
			tex_mode=(tex_mode+1) % (SPHERE+1);
			glutPostRedisplay();
			break;
	}
}

void init_model() {
	int i;
	GLubyte *image;
	
	for(i=0; i<n_textures; i++) {
		image=glmReadPPM(textures[i].filename, &(textures[i].w), &(textures[i].h));
	
		glGenTextures(1, &(textures[i].texName));
		glBindTexture(GL_TEXTURE_2D, textures[i].texName);
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
		
		gluBuild2DMipmaps( GL_TEXTURE_2D, 3, textures[i].w, textures[i].h, GL_RGB, GL_UNSIGNED_BYTE, image );
	}
}
	

