/*
 *  TexUtils.c
 *  22_3_06
 *
 *  Created by Simone Portuesi on 21/03/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

/* Include GL, GLU & GLUT */
#ifdef WIN32
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>
#include <GLUT/glut.h>
#endif

#ifdef linux
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#endif

/* Include some standard stuff */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "TexUtils.h"

/* glmReadPPM: read a PPM raw (type P6) file.  The PPM file has a header
 * that should look something like:
 *
 *    P6
 *    # comment
 *    width height max_value
 *    rgbrgbrgb...
 *
 * where "P6" is the magic cookie which identifies the file type and
 * should be the only characters on the first line followed by a
 * carriage return.  Any line starting with a # mark will be treated
 * as a comment and discarded.   After the magic cookie, three integer
 * values are expected: width, height of the image and the maximum
 * value for a pixel (max_value must be < 256 for PPM raw files).  The
 * data section consists of width*height rgb triplets (one byte each)
 * in binary format (i.e., such as that written with fwrite() or
 * equivalent).
 *
 * The rgb data is returned as an array of unsigned chars (packed
 * rgb).  The malloc()'d memory should be free()'d by the caller.  If
 * an error occurs, an error message is sent to stderr and NULL is
 * returned.
 *
 * filename   - name of the .ppm file.
 * width      - will contain the width of the image on return.
 * height     - will contain the height of the image on return.
 *
 */
GLubyte* glmReadPPM(char* filename, int* width, int* height)
{
    FILE* fp;
    int i, w, h, d;
    unsigned char* image;
    char head[70];          /* max line <= 70 in PPM (per spec). */
    
    fp = fopen(filename, "rb");
    if (!fp) {
        perror(filename);
        return NULL;
    }
    
    /* grab first two chars of the file and make sure that it has the
       correct magic cookie for a raw PPM file. */
    fgets(head, 70, fp);
    if (strncmp(head, "P6", 2)) {
        fprintf(stderr, "%s: Not a raw PPM file\n", filename);
		fclose(fp);
        return NULL;
    }
    
    /* grab the three elements in the header (width, height, maxval). */
    i = 0;
    while(i < 3) {
        fgets(head, 70, fp);
        if (head[0] == '#')     /* skip comments. */
            continue;
        if (i == 0)
            i += sscanf(head, "%d %d %d", &w, &h, &d);
        else if (i == 1)
            i += sscanf(head, "%d %d", &h, &d);
        else if (i == 2)
            i += sscanf(head, "%d", &d);
    }
    
    /* Allocate memory */
    image = (unsigned char*)malloc(sizeof(unsigned char)*w*h*3);
	if(!image) {
		fprintf(stderr, "Memory Allocation Error\n");
		fclose(fp);
        return NULL;
    }
	
	
	/* grab all the image data in one fell swoop. */
    fread(image, sizeof(GLubyte), w*h*3, fp);
    fclose(fp);
    
    *width = w;
    *height = h;
    return image;
}

GLubyte* make2DCheckImage(int w, int h, int step) {
	GLubyte fgcolor[4] = { 255, 255, 0, 255 };
	GLubyte bgcolor[4] = { 255, 0, 0 ,255};
	int i,j;
	int clen=4;
	GLubyte* image;
	
	
	/* Allocate memory */
    image = (unsigned char*)malloc(sizeof(GLubyte)*w*h*clen);
	if(!image) {
		fprintf(stderr, "Memory Allocation Error\n");
        return NULL;
    }
	
	/* Make checkboard pattern */
	for(i=0;i<w;i++) {
		for(j=0;j<h;j++) {
			if(((i / step) % 2) ^ ((j / step) % 2)) memcpy(image+(i*w+j)*clen,fgcolor,clen);
			else memcpy(image+(i*w+j)*clen,bgcolor,clen);
		}
	}
	
	return image;
}
	